package com.jz.dm.common.util;

import com.alibaba.fastjson.PropertyNamingStrategy;
import com.jz.dm.common.exception.SignatureException;
import com.jz.dm.model.signtype.Md5;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.*;

/**
 * 签名工具
 *
 */
@Slf4j
public class SignatureUtil {

    public static final String SIGN_TYPE_RSA             = "RSA";

    /**
     * sha256WithRsa 算法请求类型
     */
    public static final String SIGN_TYPE_RSA2            = "RSA2";

    public static final String SIGN_ALGORITHMS           = "SHA1WithRSA";

    public static final String SIGN_SHA256RSA_ALGORITHMS = "SHA256WithRSA";

    public static final String SIGN_REQUEST              = "SIGN_REQUEST";

    public static final String SIGN_CALLBACK             = "SIGN_CALLBACK";

    /**
     * 获取签名校验内容
     * 
     * @param params
     * @return
     */
    private static String getVerifyContent(Map<String, Object> params, String... excludeParams) {
        if (params == null) {
            return null;
        }

        if (excludeParams == null || excludeParams.length == 0) {
            params.remove("sign");
        } else {
            for (String excludeParam : excludeParams) {
                params.remove(excludeParam);
            }
        }

        StringBuffer content = new StringBuffer();
        List<String> keys = new ArrayList<String>(params.keySet());
        Collections.sort(keys);

        for (String key : keys) {
            if (params.get(key) != null){
                String value = params.get(key).toString();
                if (!StringUtils.isEmpty(value)) {
                    content.append(key).append("=").append(value);
                }
            }
        }

        return content.toString();
    }

    /**
     * 校验签名
     * 
     * @param params
     * @param publicKey
     * @param charset
     * @return
     * @throws
     */
    public static boolean verify(Map params, String publicKey, String charset,
                                 SignType signType, String... excludeParams) {
        String sign = (String) params.get("sign");
        String content = getVerifyContent(params, excludeParams);

        log.info("请求验签: {}", content);
        if (signType == SignType.RSA) {
            return rsaVerify(content, sign, publicKey, charset);
        } else if (signType == SignType.RSA2) {
            return rsa256Verify(content, sign, publicKey, charset);
        } else if (signType == SignType.MD5) {
            String calcSign = Md5.encrypt(content, publicKey, charset);
            return StringUtils.equals(calcSign, sign);
        } else {
            throw new SignatureException(OpenApiResultCode.SIGN_TYPE_NOT_SUPPORT,
                "Sign Type is Not Support : signType=" + signType);
        }
    }

    public static void main(String[] args) {
        //ed123fca54c919bce4d5e16b6ebd0304
        Map<String,Object> map = new HashMap<>();

        Map<String,Object> map1 = new HashMap<>();

        map1.put("tradeNo", "2020041510842580008");
        map1.put("outTradeNo", "XJ1586919520927");
        map1.put("status", "PAYMENT");

        map.put("data", map1);

        String responseJson = JsonUtil.toJSONString(map,
                PropertyNamingStrategy.SnakeCase);
        String s = "app_id=200407008243charset=UTF-8method=trade.submitparams={\"accountType\":\"WBANK\",\"outTradeNo\":\"TS2020041411353338464\",\"remark\":\"test11\",\"items\":[{\"amount\":\"10\",\"idCard\":\"430524199707287172\",\"mobile\":\"15910401066\",\"name\":\"马斌\",\"remark\":\"test22\",\"cardNo\":\"6217995550000488697\"}]}sign_type=MD5timestamp=1586858028000version=1.0.0";
        SignType signType = SignType.MD5;
        String calcSign = Md5.encrypt(s, "pwxw5FaQ4UFvycYfbmRvqPQ1m96aJblURXyVsnMXPe8qJhurmebv1XPnvPkJjjC8", "UTF-8");
        System.out.println(calcSign);
    }

    /**
     * rsa校验签名
     * 
     * @param content
     * @param sign
     * @param publicKey
     * @param charset
     * @return
     * @throws
     */
    public static boolean rsaVerify(String content, String sign, String publicKey, String charset) {
        try {
            PublicKey pubKey = getPublicKeyFromX509("RSA",
                new ByteArrayInputStream(publicKey.getBytes()));

            java.security.Signature signature = java.security.Signature
                .getInstance(SIGN_ALGORITHMS);

            signature.initVerify(pubKey);

            if (StringUtil.isEmpty(charset)) {
                signature.update(content.getBytes());
            } else {
                signature.update(content.getBytes(charset));
            }
            return signature.verify(Base64.decodeBase64(sign.getBytes()));
        } catch (Exception e) {
            throw new SignatureException(OpenApiResultCode.SIGN_VERIFY_ERROR,
                "RSAcontent = " + content + "; charset = " + charset, e);
        }
    }

    /**
     * rsa256校验签名
     *
     * @param content
     * @param sign
     * @param publicKey
     * @param charset
     * @return
     * @throws
     */
    public static boolean rsa256Verify(String content, String sign, String publicKey,
                                       String charset) {
        try {
            PublicKey pubKey = getPublicKeyFromX509("RSA",
                new ByteArrayInputStream(publicKey.getBytes()));

            java.security.Signature signature = java.security.Signature
                .getInstance(SIGN_SHA256RSA_ALGORITHMS);

            signature.initVerify(pubKey);

            if (StringUtil.isEmpty(charset)) {
                signature.update(content.getBytes());
            } else {
                signature.update(content.getBytes(charset));
            }
            return signature.verify(Base64.decodeBase64(sign.getBytes()));
        } catch (Exception e) {
            throw new SignatureException(OpenApiResultCode.SIGN_VERIFY_ERROR,
                "RSAcontent = " + content + "; charset = " + charset, e);
        }
    }

    /**
     * 获取公钥
     *
     * @param algorithm
     * @param ins
     * @return
     * @throws Exception
     */
    public static PublicKey getPublicKeyFromX509(String algorithm,
                                                 InputStream ins) throws Exception {
        KeyFactory keyFactory = KeyFactory.getInstance(algorithm);

        StringWriter writer = new StringWriter();
        StreamUtil.io(new InputStreamReader(ins), writer);

        byte[] encodedKey = writer.toString().getBytes();

        encodedKey = Base64.decodeBase64(encodedKey);

        return keyFactory.generatePublic(new X509EncodedKeySpec(encodedKey));
    }

    /**
     * 获取签名内容
     *
     * @param params
     * @return
     */
    public static String getSignContent(Map<String, Object> params) {
        StringBuffer content = new StringBuffer();
        List<String> keys = new ArrayList<String>(params.keySet());
        Collections.sort(keys);
        int index = 0;
        for (int i = 0; i < keys.size(); i++) {
            String key = keys.get(i);
            String value = params.get(key) == null ? "" : params.get(key).toString();
            if (StringUtil.isNotEmpty(key, value)) {
                content.append(key + "=" + value);
                index++;
            }
        }
        return content.toString();
    }

    /**
     * 内容签名
     *
     * @param params
     * @param publicKey
     * @param charset
     * @param signType
     * @param providePublicKey true : 惠多多提供外部公钥了， false ： 没有对外提供公钥，使用外部公钥加密做签名
     * @return
     */
    public static String sign(Map<String, Object> params, String publicKey, String charset,
                              SignType signType, String businessType, boolean providePublicKey) {
        String signContent = getSignContent(params);
        return sign(signContent, publicKey, charset, signType, businessType, providePublicKey);
    }

    /**
     *  rsa内容签名
     *
     * @param content
     * @param privateKey
     * @param charset
     * @return
     */
    public static String sign(String content, String privateKey, String charset,
                              SignType signType, String type, boolean providePublicKey) {
        log.info("参与加签字符串：{}", content);
        if (signType == SignType.RSA) {
            if (SIGN_CALLBACK.equals(type)) {
                if (providePublicKey) {
                    return callBackRsaSignByPrivateKey(content, privateKey, charset);
                } else {
                    return callBackRsaSignByPublicKey(content, privateKey, charset);
                }
            }
            if (providePublicKey) {
                return rsaSignByPrivateKey(content, privateKey, charset);
            } else {
                return rsaSignByPublicKey(content, privateKey, charset);
            }
        } else if (signType == SignType.RSA2) {
            return rsa256Sign(content, privateKey, charset);
        } else if (signType == SignType.MD5) {
            return Md5.encrypt(content, privateKey, charset);
        } else {
            throw new SignatureException(OpenApiResultCode.SIGN_TYPE_NOT_SUPPORT,
                "Sign Type is Not Support : signType=" + signType);
        }
    }

    public static PrivateKey getPrivateKeyFromPKCS8(String algorithm,
                                                    InputStream ins) throws Exception {
        if (ins == null || StringUtil.isEmpty(algorithm)) {
            return null;
        }

        KeyFactory keyFactory = KeyFactory.getInstance(algorithm);

        byte[] privateKey = StreamUtil.readText(ins).getBytes();

        privateKey = Base64.decodeBase64(privateKey);

        return keyFactory.generatePrivate(new PKCS8EncodedKeySpec(privateKey));
    }


    /**
     * sha256WithRsa 加签
     *
     * @param content
     * @param privateKey
     * @param charset
     * @return
     */
    public static String rsa256Sign(String content, String privateKey, String charset) {

        try {
            PrivateKey priKey = getPrivateKeyFromPKCS8(SIGN_TYPE_RSA,
                new ByteArrayInputStream(privateKey.getBytes()));

            java.security.Signature signature = java.security.Signature
                .getInstance(SIGN_SHA256RSA_ALGORITHMS);

            signature.initSign(priKey);

            if (StringUtil.isEmpty(charset)) {
                signature.update(content.getBytes());
            } else {
                signature.update(content.getBytes(charset));
            }

            byte[] signed = signature.sign();

            return new String(Base64.encodeBase64(signed));
        } catch (Exception e) {
            throw new SignatureException(OpenApiResultCode.DATA_SIGN_ERROR,
                "RSAcontent = " + content + "; charset = " + charset, e);
        }

    }

    /**
     * sha1WithRsa 加签
     * 
     * @param content
     * @param publicKey
     * @param charset
     * @return
     */
    public static String rsaSignByPublicKey(String content, String publicKey, String charset) {
        try {
            byte[] signed = RSA.encryptByPublicKey(content.getBytes(charset), publicKey);
            return new String(Base64.encodeBase64(signed));
        } catch (InvalidKeySpecException e) {
            throw new SignatureException(OpenApiResultCode.PUBLIC_KEY_FORMAT_ERROR,
                "RSA公钥格式不正确，请检查是否正确配置了PKCS8格式的公钥", e);
        } catch (Exception e) {
            throw new SignatureException(OpenApiResultCode.DATA_SIGN_ERROR,
                "RSAcontent = " + content + "; charset = " + charset, e);
        }
    }

    /**
     * sha1WithRsa 加签
     *
     * @param content
     * @param privateKey
     * @param charset
     * @return
     */
    public static String rsaSignByPrivateKey(String content, String privateKey, String charset) {
        try {
            PrivateKey priKey = getPrivateKeyFromPKCS8(SIGN_TYPE_RSA,
                    new ByteArrayInputStream(privateKey.getBytes()));

            java.security.Signature signature = java.security.Signature
                    .getInstance(SIGN_ALGORITHMS);

            signature.initSign(priKey);

            if (StringUtil.isEmpty(charset)) {
                signature.update(content.getBytes());
            } else {
                signature.update(content.getBytes(charset));
            }

            byte[] signed = signature.sign();

            return new String(Base64.encodeBase64(signed));
        } catch (InvalidKeySpecException e) {
            throw new SignatureException(OpenApiResultCode.PRIVATE_KEY_FORMAT_ERROR,
                    "RSA私钥格式不正确，请检查是否正确配置了PKCS8格式的私钥", e);
        } catch (Exception e) {
            throw new SignatureException(OpenApiResultCode.DATA_SIGN_ERROR,
                    "RSAcontent = " + content + "; charset = " + charset, e);
        }
    }

    /**
     * sha1WithRsa 回调内容加签
     * @param content
     * @param publicKey
     * @param charset
     * @return
     */
    public static String callBackRsaSignByPublicKey(String content, String publicKey, String charset) {
        try {
            byte[] signed = RSA.encryptByPublicKey(Md5.encrypt(content,"").getBytes(charset), publicKey);
            return new String(Base64.encodeBase64(signed));
        } catch (InvalidKeySpecException e) {
            throw new SignatureException(OpenApiResultCode.PUBLIC_KEY_FORMAT_ERROR,
                    "RSA公钥格式不正确，请检查是否正确配置了PKCS8格式的公钥", e);
        } catch (Exception e) {
            throw new SignatureException(OpenApiResultCode.DATA_SIGN_ERROR,
                    "RSAcontent = " + content + "; charset = " + charset, e);
        }
    }

    /**
     * sha1WithRsa 回调内容加签
     * @param content
     * @param privateKey
     * @param charset
     * @return
     */
    public static String callBackRsaSignByPrivateKey(String content, String privateKey, String charset) {
        try {
            PrivateKey priKey = getPrivateKeyFromPKCS8(SIGN_TYPE_RSA,
                    new ByteArrayInputStream(privateKey.getBytes()));

            java.security.Signature signature = java.security.Signature
                    .getInstance(SIGN_ALGORITHMS);

            signature.initSign(priKey);

            if (StringUtil.isEmpty(charset)) {
                signature.update(Md5.encrypt(content,"").getBytes());
            } else {
                signature.update(Md5.encrypt(content,"").getBytes(charset));
            }

            byte[] signed = signature.sign();

            return new String(Base64.encodeBase64(signed));
        } catch (InvalidKeySpecException e) {
            throw new SignatureException(OpenApiResultCode.PRIVATE_KEY_FORMAT_ERROR,
                    "RSA私钥格式不正确，请检查是否正确配置了PKCS8格式的私钥", e);
        } catch (Exception e) {
            throw new SignatureException(OpenApiResultCode.DATA_SIGN_ERROR,
                    "RSAcontent = " + content + "; charset = " + charset, e);
        }
    }
}
