package com.jz.dm.gateway;

import com.alibaba.fastjson.JSON;
import com.jz.dm.common.exception.OpenApiException;
import com.jz.dm.common.util.OpenApiRequest;
import com.jz.dm.common.util.OpenApiResponse;
import com.jz.dm.common.util.OpenApiResultCode;
import com.jz.dm.common.util.StringUtil;
import com.jz.dm.model.DispatchContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 默认openapi分发器
 *
 * @author key
 */
@Component("defaultOpenApiDispatcher")
public class DefaultOpenApiDispatcher implements OpenApiDispatcher {

    private static final Logger LOGGER    = LoggerFactory
        .getLogger(DefaultOpenApiDispatcher.class);

    /**
     * openapi服务
     */
    private final Map<String, OpenApiService> openApiServices = new ConcurrentHashMap<String, OpenApiService>();

    /**
     * 服务注入可参考: /openapi-demo/src/main/resources/META-INF/application-context.xml
     * @param openApiServiceList 
     * <pre>
     * &lt;!-- 定义开放接口服务 -->
     * &lt;bean id="addUserOpenApiService" class="com.xxx.openapi.demo.openapi.AddUserOpenApiService" /&gt;
     * &lt;bean id="getUserOpenApiService" class="com.xxx.openapi.demo.openapi.GetUserOpenApiService" /&gt;
     * </pre>
     */
    @Resource
    public void setOpenApiServices(List<OpenApiService> openApiServiceList) {
        for (OpenApiService openApiService : openApiServiceList) {
            String openApiServiceKey = getOpenApiServiceKey(openApiService.getOpenApiMethod(),
                openApiService.getOpenApiVersion());
            openApiServices.put(openApiServiceKey, openApiService);
        }
    }

    private String getOpenApiServiceKey(String openApiMethod, String openApiVersion) {
        return openApiMethod + "&" + openApiVersion;
    }


    @Override
    public String doDispatch(DispatchContext context) {
        OpenApiResponse response = new OpenApiResponse();
        String openApiServiceKey = getOpenApiServiceKey(context.getOpenApiMethod(),
            context.getOpenApiVersion());
        OpenApiService openApiService = openApiServices.get(openApiServiceKey);
        if (openApiService == null) {
            response.setCode(OpenApiResultCode.ILLEGAL_INTERFACE.getCode());
            response.setMsg(OpenApiResultCode.ILLEGAL_INTERFACE.getMsg());
            return JSON.toJSONString(response.getAttributes());
        }

        try {
            OpenApiRequest request = new OpenApiRequest(context.getOpenApiParams());
            request.setAppId(context.getAppId());
            request.setExtAttributes(context.getExtAttributes());
            openApiService.doService(request, response);
        } catch (Throwable ex) {
            if (ex instanceof OpenApiException) {
                OpenApiException oae = (OpenApiException) ex;
                response.setCode(oae.getResultCode().getCode());
                response.setMsg(oae.getResultCode().getMsg());
            } else {
                LOGGER.error("OpenApiService doService error,DispatchContext=" + context, ex);
                response.setCode(OpenApiResultCode.UNKNOWN_EXCEPTION.getCode());
                response.setMsg(OpenApiResultCode.UNKNOWN_EXCEPTION.getMsg());
            }
            return JSON.toJSONString(response.getAttributes());
        }
        if (StringUtil.isEmpty(response.getCode())) {
            response.setCode(OpenApiResultCode.SUCCESS.getCode());
            response.setMsg(OpenApiResultCode.SUCCESS.getMsg());
        }
        return JSON.toJSONString(response.getAttributes());
    }

}
